@@ -1,11 +1,17 @@
+from __future__ import annotations
 import time
 import logging
 import json
 import pathlib
-from typing import Optional
+from typing import Optional, Dict, Any, TYPE_CHECKING
+# https://stackoverflow.com/a/39757388/929999
+if TYPE_CHECKING:
+	from .blockdevice import BlockDevice
+	_: Any
+
 from .partition import Partition
 from .validators import valid_fs_type
-from ..exceptions import DiskError
+from ..exceptions import DiskError, SysCallError
 from ..general import SysCommand
 from ..output import log
 from ..storage import storage
@@ -13,31 +19,37 @@ from ..storage import storage
 GPT = 0b00000001
 MBR = 0b00000010
 
+# A sane default is 5MiB, that allows for plenty of buffer for GRUB on MBR
+# but also 4MiB for memory cards for instance. And another 1MiB to avoid issues.
+# (we've been pestered by disk issues since the start, so please let this be here for a few versions)
+DEFAULT_PARTITION_START = '5MiB'
+
 class Filesystem:
 	# TODO:
 	#   When instance of a HDD is selected, check all usages and gracefully unmount them
 	#   as well as close any crypto handles.
-	def __init__(self, blockdevice, mode):
+	def __init__(self, blockdevice :BlockDevice, mode :int):
 		self.blockdevice = blockdevice
 		self.mode = mode
 
-	def __enter__(self, *args, **kwargs):
+	def __enter__(self, *args :str, **kwargs :str) -> 'Filesystem':
 		return self
 
-	def __repr__(self):
+	def __repr__(self) -> str:
 		return f"Filesystem(blockdevice={self.blockdevice}, mode={self.mode})"
 
-	def __exit__(self, *args, **kwargs):
+	def __exit__(self, *args :str, **kwargs :str) -> bool:
 		# TODO: https://stackoverflow.com/questions/28157929/how-to-safely-handle-an-exception-inside-a-context-manager
 		if len(args) >= 2 and args[1]:
 			raise args[1]
+
 		SysCommand('sync')
 		return True
 
-	def partuuid_to_index(self, uuid):
+	def partuuid_to_index(self, uuid :str) -> Optional[int]:
 		for i in range(storage['DISK_RETRY_ATTEMPTS']):
 			self.partprobe()
-			time.sleep(storage['DISK_TIMEOUTS'] * i)
+			time.sleep(max(0.1, storage['DISK_TIMEOUTS'] * i))
 
 			# We'll use unreliable lbslk to grab children under the /dev/<device>
 			output = json.loads(SysCommand(f"lsblk --json {self.blockdevice.device}").decode('UTF-8'))
@@ -51,14 +63,11 @@ class Filesystem:
 
 		raise DiskError(f"Failed to convert PARTUUID {uuid} to a partition index number on blockdevice {self.blockdevice.device}")
 
-	def has_partition_table(self) -> bool:
-		return bool(str(SysCommand(f'blkid -s PTTYPE -o value {self.blockdevice.path}')))
-
-	def load_layout(self, layout :dict):
+	def load_layout(self, layout :Dict[str, Any]) -> None:
 		from ..luks import luks2
 
 		# If the layout tells us to wipe the drive, we do so
-		if layout.get('wipe', False) or self.has_partition_table() is False:
+		if layout.get('wipe', False):
 			if self.mode == GPT:
 				if not self.parted_mklabel(self.blockdevice.device, "gpt"):
 					raise KeyError(f"Could not create a GPT label on {self}")
@@ -68,20 +77,26 @@ class Filesystem:
 
 			self.blockdevice.flush_cache()
 
+		prev_partition = None
 		# We then iterate the partitions in order
 		for partition in layout.get('partitions', []):
-			# Create a new partition if a filesystem is defined in 'format'
-			if partition.get('filesystem', {}).get('format', False) and not partition.get('PARTUUID', None):
+			# We don't want to re-add an existing partition (those containing a UUID already)
+			if partition.get('wipe', False) and not partition.get('PARTUUID', None):
+				print(_("Adding partition...."))
+				start = partition.get('start') or (
+					prev_partition and f'{prev_partition["device_instance"].end_sectors}s' or DEFAULT_PARTITION_START)
 				partition['device_instance'] = self.add_partition(partition.get('type', 'primary'),
-																	start=partition.get('start', '1MiB'), # TODO: Revisit sane block starts (4MB for memorycards for instance)
+																	start=start,
 																	end=partition.get('size', '100%'),
 																	partition_format=partition.get('filesystem', {}).get('format', 'btrfs'))
-			# Or re-use existing partition if we can find PARTUUID
+				# TODO: device_instance some times become None
+				# print('Device instance:', partition['device_instance'])
+
 			elif (partition_uuid := partition.get('PARTUUID')) and (partition_instance := self.blockdevice.get_partition(uuid=partition_uuid)):
-				log(f"[BETA] Re-using partition_instance: {partition_instance}", level=logging.WARNING, fg="yellow")
+				print(_("Re-using partition instance: {}").format(partition_instance))
 				partition['device_instance'] = partition_instance
 			else:
-				raise ValueError(f"{self}.load_layout() doesn't know how to handle {partition}. Cannot continue without a new partition with 'format' definition or a UUID ({partition.get('PARTUUID')}) to re-use a partition.")
+				raise ValueError(f"{self}.load_layout() doesn't know how to continue without a new partition definition or a UUID ({partition.get('PARTUUID')}) on the device ({self.blockdevice.get_partition(uuid=partition.get('PARTUUID'))}).")
 
 			if partition.get('filesystem', {}).get('format', False):
 
@@ -97,22 +112,21 @@ class Filesystem:
 								raise ValueError(f"Missing encryption password for {partition['device_instance']}")
 
 							from ..user_interaction import get_password
-							storage['arguments']['!encryption-password'] = get_password(f"Enter a encryption password for {partition['device_instance']}")
+
+							prompt = str(_('Enter a encryption password for {}').format(partition['device_instance']))
+							storage['arguments']['!encryption-password'] = get_password(prompt)
 
 						partition['!password'] = storage['arguments']['!encryption-password']
-					
-					# to be able to generate an unique name in case the partition will not be mounted
+
 					if partition.get('mountpoint',None):
-						ppath = partition['mountpoint']
+						loopdev = f"{storage.get('ENC_IDENTIFIER', 'ai')}{pathlib.Path(partition['mountpoint']).name}loop"
 					else:
-						ppath = partition['device_instance'].path
-					loopdev = f"{storage.get('ENC_IDENTIFIER', 'ai')}{pathlib.Path(ppath).name}loop"
+						loopdev = f"{storage.get('ENC_IDENTIFIER', 'ai')}{pathlib.Path(partition['device_instance'].path).name}"
 
 					partition['device_instance'].encrypt(password=partition['!password'])
-
 					# Immediately unlock the encrypted device to format the inner volume
 					with luks2(partition['device_instance'], loopdev, partition['!password'], auto_unmount=True) as unlocked_device:
-						if not partition.get('format'):
+						if not partition.get('wipe'):
 							if storage['arguments'] == 'silent':
 								raise ValueError(f"Missing fs-type to format on newly created encrypted partition {partition['device_instance']}")
 							else:
@@ -123,12 +137,12 @@ class Filesystem:
 									while True:
 										partition['filesystem']['format'] = input(f"Enter a valid fs-type for newly encrypted partition {partition['filesystem']['format']}: ").strip()
 										if not partition['filesystem']['format'] or valid_fs_type(partition['filesystem']['format']) is False:
-											print("You need to enter a valid fs-type in order to continue. See `man parted` for valid fs-type's.")
+											print(_("You need to enter a valid fs-type in order to continue. See `man parted` for valid fs-type's."))
 											continue
 										break
 
-						unlocked_device.format(partition['filesystem']['format'], options=partition.get('options', []))
-				elif partition.get('format', False):
+						unlocked_device.format(partition['filesystem']['format'], options=format_options)
+				elif partition.get('wipe', False):
 					if not partition['device_instance']:
 						raise DiskError(f"Internal error caused us to loose the partition. Please report this issue upstream!")
 
@@ -138,27 +152,29 @@ class Filesystem:
 				log(f"Marking partition {partition['device_instance']} as bootable.")
 				self.set(self.partuuid_to_index(partition['device_instance'].uuid), 'boot on')
 
-	def find_partition(self, mountpoint):
+			prev_partition = partition
+
+	def find_partition(self, mountpoint :str) -> Partition:
 		for partition in self.blockdevice:
 			if partition.target_mountpoint == mountpoint or partition.mountpoint == mountpoint:
 				return partition
 
 	def partprobe(self) -> bool:
-		result = SysCommand(f'partprobe {self.blockdevice.device}')
-		
-		if result.exit_code != 0:
-			log(f"Could not execute partprobe: {result!r}", level=logging.ERROR, fg="red")
-			raise DiskError(f"Could not run partprobe: {result!r}")
+		try:
+			SysCommand(f'partprobe {self.blockdevice.device}')
+		except SysCallError as error:
+			log(f"Could not execute partprobe: {error!r}", level=logging.ERROR, fg="red")
+			raise DiskError(f"Could not run partprobe on {self.blockdevice.device}: {error!r}")
 
 		return True
 
-	def raw_parted(self, string: str):
+	def raw_parted(self, string: str) -> SysCommand:
 		if (cmd_handle := SysCommand(f'/usr/bin/parted -s {string}')).exit_code != 0:
 			log(f"Parted ended with a bad exit code: {cmd_handle}", level=logging.ERROR, fg="red")
 		time.sleep(0.5)
 		return cmd_handle
 
-	def parted(self, string: str):
+	def parted(self, string: str) -> bool:
 		"""
 		Performs a parted execution of the given string
 
@@ -170,7 +186,7 @@ class Filesystem:
 		else:
 			raise DiskError(f"Parted failed to add a partition: {parted_handle}")
 
-	def use_entire_disk(self, root_filesystem_type='ext4') -> Partition:
+	def use_entire_disk(self, root_filesystem_type :str = 'ext4') -> Partition:
 		# TODO: Implement this with declarative profiles instead.
 		raise ValueError("Installation().use_entire_disk() has to be re-worked.")
 
@@ -211,7 +227,7 @@ class Filesystem:
 						raise err
 				else:
 					count += 1
-					log(f"Could not get UUID for partition. Waiting for the {count} time",level=logging.DEBUG)
+					log(f"Could not get UUID for partition. Waiting before retry attempt {count} of 10 ...",level=logging.DEBUG)
 					time.sleep(float(storage['arguments'].get('disk-sleep', 0.2)))
 			else:
 				log("Add partition is exiting due to excessive wait time", level=logging.ERROR, fg="red")
@@ -226,11 +242,11 @@ class Filesystem:
 	def set_name(self, partition: int, name: str) -> bool:
 		return self.parted(f'{self.blockdevice.device} name {partition + 1} "{name}"') == 0
 
-	def set(self, partition: int, string: str):
-		log(f"Setting '{string}'' on (parted) partition index {partition+1}", level=logging.INFO)
+	def set(self, partition: int, string: str) -> bool:
+		log(f"Setting {string} on (parted) partition index {partition+1}", level=logging.INFO)
 		return self.parted(f'{self.blockdevice.device} set {partition + 1} {string}') == 0
 
-	def parted_mklabel(self, device: str, disk_label: str):
+	def parted_mklabel(self, device: str, disk_label: str) -> bool:
 		log(f"Creating a new partition label on {device}", level=logging.INFO, fg="yellow")
 		# Try to unmount devices before attempting to run mklabel
 		try:
@@ -242,4 +258,4 @@ class Filesystem:
 		worked = self.raw_parted(f'{device} mklabel {disk_label}').exit_code == 0
 		self.partprobe()
 
-		return worked
\ No newline at end of file
+		return worked
