@@ -5,60 +5,65 @@ import logging
 import json
 import os
 import hashlib
-from typing import Optional
+from typing import Optional, Dict, Any, List, Union, Iterator
+
 from .blockdevice import BlockDevice
-from .helpers import get_mount_info, get_filesystem_type, convert_size_to_gb, split_bind_name
+from .helpers import find_mountpoint, get_filesystem_type, convert_size_to_gb, split_bind_name
 from ..storage import storage
 from ..exceptions import DiskError, SysCallError, UnknownFilesystemFormat
 from ..output import log
 from ..general import SysCommand
-
+from .btrfs import get_subvolumes_from_findmnt, BtrfsSubvolume
 
 class Partition:
-	def __init__(self, path: str, block_device: BlockDevice, part_id=None, filesystem=None, mountpoint=None, encrypted=False, autodetect_filesystem=True, auto_mount=True):
+	def __init__(self,
+		path: str,
+		block_device: BlockDevice,
+		part_id :Optional[str] = None,
+		filesystem :Optional[str] = None,
+		mountpoint :Optional[str] = None,
+		encrypted :bool = False,
+		autodetect_filesystem :bool = True):
+
 		if not part_id:
 			part_id = os.path.basename(path)
 
 		self.block_device = block_device
+		if type(self.block_device) is str:
+			raise ValueError(f"Partition()'s 'block_device' parameter has to be a archinstall.BlockDevice() instance!")
+
 		self.path = path
 		self.part_id = part_id
-		self.mountpoint = mountpoint
 		self.target_mountpoint = mountpoint
 		self.filesystem = filesystem
 		self._encrypted = None
 		self.encrypted = encrypted
 		self.allow_formatting = False
 
-		if mountpoint and auto_mount is True:
+		if mountpoint:
 			self.mount(mountpoint)
 
 		try:
-			mount_information = get_mount_info(self.path)
+			self.mount_information = list(find_mountpoint(self.path))
 		except DiskError:
-			mount_information = {}
-
-		if mount_information.get('target', None):
-			if self.mountpoint != mount_information.get('target', None) and mountpoint:
-				raise DiskError(f"{self} was given a mountpoint but the actual mountpoint differs: {mount_information.get('target', None)}")
-
-			if target := mount_information.get('target', None):
-				self.mountpoint = target
+			self.mount_information = [{}]
 
 		if not self.filesystem and autodetect_filesystem:
-			if fstype := mount_information.get('fstype', get_filesystem_type(path)):
-				self.filesystem = fstype
+			self.filesystem = get_filesystem_type(path)
 
 		if self.filesystem == 'crypto_LUKS':
 			self.encrypted = True
 
-	def __lt__(self, left_comparitor):
+	def __lt__(self, left_comparitor :BlockDevice) -> bool:
 		if type(left_comparitor) == Partition:
 			left_comparitor = left_comparitor.path
 		else:
 			left_comparitor = str(left_comparitor)
-		return self.path < left_comparitor  # Not quite sure the order here is correct. But /dev/nvme0n1p1 comes before /dev/nvme0n1p5 so seems correct.
 
-	def __repr__(self, *args, **kwargs):
+		# The goal is to check if /dev/nvme0n1p1 comes before /dev/nvme0n1p5
+		return self.path < left_comparitor
+
+	def __repr__(self, *args :str, **kwargs :str) -> str:
 		mount_repr = ''
 		if self.mountpoint:
 			mount_repr = f", mounted={self.mountpoint}"
@@ -70,10 +75,10 @@ class Partition:
 		else:
 			return f'Partition(path={self.path}, size={self.size}, PARTUUID={self._safe_uuid}, fs={self.filesystem}{mount_repr})'
 
-	def __dump__(self):
+	def __dump__(self) -> Dict[str, Any]:
 		return {
 			'type': 'primary',
-			'PARTUUID': self.uuid,
+			'PARTUUID': self._safe_uuid,
 			'wipe': self.allow_formatting,
 			'boot': self.boot,
 			'ESP': self.boot,
@@ -87,14 +92,28 @@ class Partition:
 		}
 
 	@property
-	def sector_size(self):
+	def mountpoint(self) -> Optional[str]:
+		try:
+			data = json.loads(SysCommand(f"findmnt --json -R {self.path}").decode())
+			for filesystem in data['filesystems']:
+				return filesystem.get('target')
+
+		except SysCallError as error:
+			# Not mounted anywhere most likely
+			log(f"Could not locate mount information for {self.path}: {error}", level=logging.DEBUG)
+			pass
+
+		return None
+
+	@property
+	def sector_size(self) -> Optional[int]:
 		output = json.loads(SysCommand(f"lsblk --json -o+LOG-SEC {self.device_path}").decode('UTF-8'))
 
 		for device in output['blockdevices']:
 			return device.get('log-sec', None)
 
 	@property
-	def start(self):
+	def start(self) -> Optional[str]:
 		output = json.loads(SysCommand(f"sfdisk --json {self.block_device.path}").decode('UTF-8'))
 
 		for partition in output.get('partitiontable', {}).get('partitions', []):
@@ -102,7 +121,8 @@ class Partition:
 				return partition['start']  # * self.sector_size
 
 	@property
-	def end(self):
+	def end(self) -> Optional[str]:
+		# TODO: actually this is size in sectors unit
 		# TODO: Verify that the logic holds up, that 'size' is the size without 'start' added to it.
 		output = json.loads(SysCommand(f"sfdisk --json {self.block_device.path}").decode('UTF-8'))
 
@@ -111,23 +131,32 @@ class Partition:
 				return partition['size']  # * self.sector_size
 
 	@property
-	def size(self):
+	def end_sectors(self) -> Optional[str]:
+		output = json.loads(SysCommand(f"sfdisk --json {self.block_device.path}").decode('UTF-8'))
+
+		for partition in output.get('partitiontable', {}).get('partitions', []):
+			if partition['node'] == self.path:
+				return partition['start'] + partition['size']
+
+	@property
+	def size(self) -> Optional[float]:
 		for i in range(storage['DISK_RETRY_ATTEMPTS']):
 			self.partprobe()
+			time.sleep(max(0.1, storage['DISK_TIMEOUTS'] * i))
 
-			if (handle := SysCommand(f"lsblk --json -b -o+SIZE {self.device_path}")).exit_code == 0:
-				lsblk = json.loads(handle.decode('UTF-8'))
+			try:
+				lsblk = json.loads(SysCommand(f"lsblk --json -b -o+SIZE {self.device_path}").decode())
 
 				for device in lsblk['blockdevices']:
 					return convert_size_to_gb(device['size'])
-			elif handle.exit_code == 8192:
-				# Device is not a block device
-				return None
-
-			time.sleep(storage['DISK_TIMEOUTS'])
+			except SysCallError as error:
+				if error.exit_code == 8192:
+					return None
+				else:
+					raise error
 
 	@property
-	def boot(self):
+	def boot(self) -> bool:
 		output = json.loads(SysCommand(f"sfdisk --json {self.block_device.path}").decode('UTF-8'))
 
 		# Get the bootable flag from the sfdisk output:
@@ -142,12 +171,13 @@ class Partition:
 
 		for partition in output.get('partitiontable', {}).get('partitions', []):
 			if partition['node'] == self.path:
-				return partition.get('bootable', False)
+				# first condition is for MBR disks, second for GPT disks
+				return partition.get('bootable', False) or partition.get('type','') == 'C12A7328-F81F-11D2-BA4B-00A0C93EC93B'
 
 		return False
 
 	@property
-	def partition_type(self):
+	def partition_type(self) -> Optional[str]:
 		lsblk = json.loads(SysCommand(f"lsblk --json -o+PTTYPE {self.device_path}").decode('UTF-8'))
 
 		for device in lsblk['blockdevices']:
@@ -162,14 +192,16 @@ class Partition:
 		For bind mounts all the subvolumes share the same uuid
 		"""
 		for i in range(storage['DISK_RETRY_ATTEMPTS']):
-			self.partprobe()
-			time.sleep(storage['DISK_TIMEOUTS'] * i)
+			if not self.partprobe():
+				raise DiskError(f"Could not perform partprobe on {self.device_path}")
+				
+			time.sleep(max(0.1, storage['DISK_TIMEOUTS'] * i))
 
 			partuuid = self._safe_uuid
 			if partuuid:
 				return partuuid
 
-		raise DiskError(f"Could not get PARTUUID for {self.path} using 'lsblk -J -o+PARTUUID {self.path}'")
+		raise DiskError(f"Could not get PARTUUID for {self.path} using 'blkid -s PARTUUID -o value {self.path}'")
 
 	@property
 	def _safe_uuid(self) -> Optional[str]:
@@ -178,23 +210,35 @@ class Partition:
 		This function should only be used where uuid is not crucial.
 		For instance when you want to get a __repr__ of the class.
 		"""
-		self.partprobe()
-		return SysCommand(f'blkid -s PARTUUID -o value {self.device_path}').decode('UTF-8').strip()
+		if not self.partprobe():
+			if self.block_device.info.get('TYPE') == 'iso9660':
+				return None
+
+			log(f"Could not reliably refresh PARTUUID of partition {self.device_path} due to partprobe error.", level=logging.DEBUG)
+
+		try:
+			return SysCommand(f'blkid -s PARTUUID -o value {self.device_path}').decode('UTF-8').strip()
+		except SysCallError as error:
+			if self.block_device.info.get('TYPE') == 'iso9660':
+				# Parent device is a Optical Disk (.iso dd'ed onto a device for instance)
+				return None
+
+			log(f"Could not get PARTUUID of partition using 'blkid -s PARTUUID -o value {self.device_path}': {error}")
 
 	@property
-	def encrypted(self):
+	def encrypted(self) -> Union[bool, None]:
 		return self._encrypted
 
 	@encrypted.setter
-	def encrypted(self, value: bool):
+	def encrypted(self, value: bool) -> None:
 		self._encrypted = value
 
 	@property
-	def parent(self):
+	def parent(self) -> str:
 		return self.real_device
 
 	@property
-	def real_device(self):
+	def real_device(self) -> str:
 		for blockdevice in json.loads(SysCommand('lsblk -J').decode('UTF-8'))['blockdevices']:
 			if parent := self.find_parent_of(blockdevice, os.path.basename(self.device_path)):
 				return f"/dev/{parent}"
@@ -202,27 +246,36 @@ class Partition:
 		return self.path
 
 	@property
-	def device_path(self):
+	def device_path(self) -> str:
 		""" for bind mounts returns the phisical path of the partition
 		"""
 		device_path, bind_name = split_bind_name(self.path)
 		return device_path
 
 	@property
-	def bind_name(self):
+	def bind_name(self) -> str:
 		""" for bind mounts returns the bind name (subvolume path).
 		Returns none if this property does not exist
 		"""
 		device_path, bind_name = split_bind_name(self.path)
 		return bind_name
 
+	@property
+	def subvolumes(self) -> Iterator[BtrfsSubvolume]:
+		for mountpoint in self.mount_information:
+			for result in get_subvolumes_from_findmnt(mountpoint):
+				yield result
+
 	def partprobe(self) -> bool:
-		if self.block_device and SysCommand(f'partprobe {self.block_device.device}').exit_code == 0:
-			time.sleep(1)
-			return True
+		try:
+			if self.block_device:
+				return 0 == SysCommand(f'partprobe {self.block_device.device}').exit_code
+		except SysCallError as error:
+			log(f"Unreliable results might be given for {self.path} due to partprobe error: {error}", level=logging.DEBUG)
+
 		return False
 
-	def detect_inner_filesystem(self, password):
+	def detect_inner_filesystem(self, password :str) -> Optional[str]:
 		log(f'Trying to detect inner filesystem format on {self} (This might take a while)', level=logging.INFO)
 		from ..luks import luks2
 
@@ -232,7 +285,7 @@ class Partition:
 		except SysCallError:
 			return None
 
-	def has_content(self):
+	def has_content(self) -> bool:
 		fs_type = get_filesystem_type(self.path)
 		if not fs_type or "swap" in fs_type:
 			return False
@@ -253,7 +306,7 @@ class Partition:
 
 		return True if files > 0 else False
 
-	def encrypt(self, *args, **kwargs):
+	def encrypt(self, *args :str, **kwargs :str) -> str:
 		"""
 		A wrapper function for luks2() instances and the .encrypt() method of that instance.
 		"""
@@ -262,7 +315,7 @@ class Partition:
 		handle = luks2(self, None, None)
 		return handle.encrypt(self, *args, **kwargs)
 
-	def format(self, filesystem=None, path=None, log_formatting=True, options=[]):
+	def format(self, filesystem :Optional[str] = None, path :Optional[str] = None, log_formatting :bool = True, options :List[str] = []) -> bool:
 		"""
 		Format can be given an overriding path, for instance /dev/null to test
 		the formatting functionality and in essence the support for the given filesystem.
@@ -294,9 +347,8 @@ class Partition:
 		elif filesystem == 'vfat':
 			options = ['-F32'] + options
 
-			mkfs = SysCommand(f"/usr/bin/mkfs.vfat {' '.join(options)} {path}").decode('UTF-8')
-			if ('mkfs.fat' not in mkfs and 'mkfs.vfat' not in mkfs) or 'command not found' in mkfs:
-				raise DiskError(f"Could not format {path} with {filesystem} because: {mkfs}")
+			if (handle := SysCommand(f"/usr/bin/mkfs.vfat {' '.join(options)} {path}")).exit_code != 0:
+				raise DiskError(f"Could not format {path} with {filesystem} because: {handle.decode('UTF-8')}")
 			self.filesystem = filesystem
 
 		elif filesystem == 'ext4':
@@ -350,7 +402,7 @@ class Partition:
 
 		return True
 
-	def find_parent_of(self, data, name, parent=None):
+	def find_parent_of(self, data :Dict[str, Any], name :str, parent :Optional[str] = None) -> Optional[str]:
 		if data['name'] == name:
 			return parent
 		elif 'children' in data:
@@ -358,18 +410,9 @@ class Partition:
 				if parent := self.find_parent_of(child, name, parent=data['name']):
 					return parent
 
-	def mount_options_has_subvolume(self, options):
-		return any(['subvol=' in x for x in options])
-
-	def mount(self, target, fs=None, options=[]):
-		if type(options) == str:
-			options = options.split(' ')
-
-		log(f"Attempting to mount {self} to {target} using options {options}", level=logging.INFO)
-
-		# Do not mount a partition that is already mounted, unless we're using subvolumes:
-		# TODO: This can be removed in favor of the new code in `master` at some point.
-		if not self.mountpoint or self.mount_options_has_subvolume(options):
+	def mount(self, target :str, fs :Optional[str] = None, options :str = '') -> bool:
+		if not self.mountpoint:
+			log(f'Mounting {self} to {target}', level=logging.INFO)
 			if not fs:
 				if not self.filesystem:
 					raise DiskError(f'Need to format (or define) the filesystem on {self} before mounting.')
@@ -379,50 +422,46 @@ class Partition:
 
 			pathlib.Path(target).mkdir(parents=True, exist_ok=True)
 
-			# If we're using bind_name (?) then append to the options if needed.
 			if self.bind_name:
 				device_path = self.device_path
-				if not self.mount_options_has_subvolume(options):
-					options.append(f"subvol={self.bind_name}")
+				# TODO options should be better be a list than a string
+				if options:
+					options = f"{options},subvol={self.bind_name}"
+				else:
+					options = f"subvol={self.bind_name}"
 			else:
 				device_path = self.path
+			try:
+				if options:
+					mnt_handle = SysCommand(f"/usr/bin/mount -t {fs_type} -o {options} {device_path} {target}")
+				else:
+					mnt_handle = SysCommand(f"/usr/bin/mount -t {fs_type} {device_path} {target}")
 
-			if options:
-				mount_options = f"-o {','.join(options)}"
-			else:
-				mount_options = ''
-
-			log(f"Mount command: /usr/bin/mount -t {fs_type} {mount_options} {device_path} {target}", fg="yellow")
-			mnt_handle = SysCommand(f"/usr/bin/mount -t {fs_type} {mount_options} {device_path} {target}")
-
-			# TODO: Should be redundant to check for exit_code
-			if mnt_handle.exit_code != 0:
-				raise DiskError(f"Could not mount {self.path} to {target} using options {options}: {mnt_handle}")
-
-			self.mountpoint = target
-			return True
-		else:
-			raise DiskError(f"Partition is already mounted to {self.mountpoint} but also with {self.bind_name}")
-
-	def unmount(self):
-		try:
-			SysCommand(f"/usr/bin/umount {self.path}")
-		except SysCallError as err:
-			exit_code = err.exit_code
-
-			# Without to much research, it seams that low error codes are errors.
-			# And above 8k is indicators such as "/dev/x not mounted.".
-			# So anything in between 0 and 8k are errors (?).
-			if 0 < exit_code < 8000:
+				# TODO: Should be redundant to check for exit_code
+				if mnt_handle.exit_code != 0:
+					raise DiskError(f"Could not mount {self.path} to {target} using options {options}")
+			except SysCallError as err:
 				raise err
 
-		self.mountpoint = None
+			return True
+
+		return False
+
+	def unmount(self) -> bool:
+		worker = SysCommand(f"/usr/bin/umount {self.path}")
+
+		# Without to much research, it seams that low error codes are errors.
+		# And above 8k is indicators such as "/dev/x not mounted.".
+		# So anything in between 0 and 8k are errors (?).
+		if 0 < worker.exit_code < 8000:
+			raise SysCallError(f"Could not unmount {self.path} properly: {worker}", exit_code=worker.exit_code)
+
 		return True
 
-	def umount(self):
+	def umount(self) -> bool:
 		return self.unmount()
 
-	def filesystem_supported(self):
+	def filesystem_supported(self) -> bool:
 		"""
 		The support for a filesystem (this partition) is tested by calling
 		partition.format() with a path set to '/dev/null' which returns two exceptions:
@@ -438,7 +477,7 @@ class Partition:
 		return True
 
 
-def get_mount_fs_type(fs):
+def get_mount_fs_type(fs :str) -> str:
 	if fs == 'ntfs':
 		return 'ntfs3'  # Needed to use the Paragon R/W NTFS driver
 	elif fs == 'fat32':
