@@ -1,3 +1,5 @@
+from typing import Optional
+
 class RequirementError(BaseException):
 	pass
 
@@ -15,12 +17,16 @@ class ProfileError(BaseException):
 
 
 class SysCallError(BaseException):
-	def __init__(self, message, exit_code):
+	def __init__(self, message :str, exit_code :Optional[int] = None) -> None:
 		super(SysCallError, self).__init__(message)
 		self.message = message
 		self.exit_code = exit_code
 
 
+class PermissionError(BaseException):
+	pass
+
+
 class ProfileNotFound(BaseException):
 	pass
 
@@ -35,3 +41,11 @@ class UserError(BaseException):
 
 class ServiceException(BaseException):
 	pass
+
+
+class PackageError(BaseException):
+	pass
+
+
+class TranslationError(BaseException):
+	pass
\ No newline at end of file
