@@ -7,7 +7,9 @@ import os
 #   (4. Added the ~/.config directory as an additional option for future reasons)
 #
 # And Keeping this in dict ensures that variables are shared across imports.
-storage = {
+from typing import Any, Dict
+
+storage: Dict[str, Any] = {
 	'PROFILE_PATH': [
 		'./profiles',
 		'~/.config/archinstall/profiles',
@@ -22,4 +24,6 @@ storage = {
 	'ENC_IDENTIFIER': 'ainst',
 	'DISK_TIMEOUTS' : 1, # seconds
 	'DISK_RETRY_ATTEMPTS' : 20, # RETRY_ATTEMPTS * DISK_TIMEOUTS is used in disk operations
+	'CMD_LOCALE':{'LC_ALL':'C'}, # default locale for execution commands. Can be overriden with set_cmd_locale()
+	'CMD_LOCALE_DEFAULT':{'LC_ALL':'C'}, # should be the same as the former. Not be used except in reset_cmd_locale()
 }
