@@ -1,5 +1,6 @@
 import logging
 import time
+from typing import Iterator
 from .exceptions import SysCallError
 from .general import SysCommand, SysCommandWorker, locate_binary
 from .installer import Installer
@@ -8,14 +9,14 @@ from .storage import storage
 
 
 class Ini:
-	def __init__(self, *args, **kwargs):
+	def __init__(self, *args :str, **kwargs :str):
 		"""
 		Limited INI handler for now.
 		Supports multiple keywords through dictionary list items.
 		"""
 		self.kwargs = kwargs
 
-	def __str__(self):
+	def __str__(self) -> str:
 		result = ''
 		first_row_done = False
 		for top_level in self.kwargs:
@@ -54,7 +55,7 @@ class Boot:
 		self.session = None
 		self.ready = False
 
-	def __enter__(self):
+	def __enter__(self) -> 'Boot':
 		if (existing_session := storage.get('active_boot', None)) and existing_session.instance != self.instance:
 			raise KeyError("Archinstall only supports booting up one instance, and a active session is already active and it is not this one.")
 
@@ -81,7 +82,7 @@ class Boot:
 		storage['active_boot'] = self
 		return self
 
-	def __exit__(self, *args, **kwargs):
+	def __exit__(self, *args :str, **kwargs :str) -> None:
 		# b''.join(sys_command('sync')) # No need to, since the underlying fs() object will call sync.
 		# TODO: https://stackoverflow.com/questions/28157929/how-to-safely-handle-an-exception-inside-a-context-manager
 
@@ -89,34 +90,40 @@ class Boot:
 			log(args[1], level=logging.ERROR, fg='red')
 			log(f"The error above occured in a temporary boot-up of the installation {self.instance}", level=logging.ERROR, fg="red")
 
-		shutdown = SysCommand(f'systemd-run --machine={self.container_name} --pty /bin/bash -c "shutdown now"')
+		shutdown = None
+
+		try:
+			shutdown = SysCommand(f'systemd-run --machine={self.container_name} --pty shutdown now')
+		except SysCallError as error:
+			if error.exit_code == 256:
+				pass
+
 		while self.session.is_alive():
 			time.sleep(0.25)
 
-		# Just one of them has to exit properly
-		if shutdown.exit_code == 0 or self.session.exit_code == 0:
+		if self.session.exit_code == 0 or (shutdown and shutdown.exit_code == 0):
 			storage['active_boot'] = None
 		else:
-			raise SysCallError(f"Could not shut down temporary boot of {self.instance} [{self.session.exit_code}/{shutdown.exit_code}]: {self.session.decode()[-200:]}", exit_code=shutdown.exit_code)
+			raise SysCallError(f"Could not shut down temporary boot of {self.instance}: {shutdown}", exit_code=shutdown.exit_code)
 
-	def __iter__(self):
+	def __iter__(self) -> Iterator[str]:
 		if self.session:
 			for value in self.session:
 				yield value
 
-	def __contains__(self, key: bytes):
+	def __contains__(self, key: bytes) -> bool:
 		if self.session is None:
 			return False
 
 		return key in self.session
 
-	def is_alive(self):
+	def is_alive(self) -> bool:
 		if self.session is None:
 			return False
 
 		return self.session.is_alive()
 
-	def SysCommand(self, cmd: list, *args, **kwargs):
+	def SysCommand(self, cmd: list, *args, **kwargs) -> SysCommand:
 		if cmd[0][0] != '/' and cmd[0][:2] != './':
 			# This check is also done in SysCommand & SysCommandWorker.
 			# However, that check is done for `machinectl` and not for our chroot command.
@@ -126,7 +133,7 @@ class Boot:
 
 		return SysCommand(["systemd-run", f"--machine={self.container_name}", "--pty", *cmd], *args, **kwargs)
 
-	def SysCommandWorker(self, cmd: list, *args, **kwargs):
+	def SysCommandWorker(self, cmd: list, *args, **kwargs) -> SysCommandWorker:
 		if cmd[0][0] != '/' and cmd[0][:2] != './':
 			cmd[0] = locate_binary(cmd[0])
 
