@@ -1,5 +1,4 @@
 # A desktop environment selector.
-
 import archinstall
 
 is_top_level_profile = True
@@ -21,6 +20,23 @@ __packages__ = [
 	'xdg-utils',
 ]
 
+__supported__ = [
+	'gnome',
+	'kde',
+	'awesome',
+	'sway',
+	'cinnamon',
+	'xfce4',
+	'lxqt',
+	'i3',
+	'bspwm',
+	'budgie',
+	'mate',
+	'deepin',
+	'enlightenment',
+	'qtile'
+]
+
 
 def _prep_function(*args, **kwargs):
 	"""
@@ -29,30 +45,15 @@ def _prep_function(*args, **kwargs):
 	other code in this stage. So it's a safe way to ask the user
 	for more input before any other installer steps start.
 	"""
-
-	supported_desktops = [
-		'gnome',
-		'kde',
-		'awesome',
-		'sway',
-		'cinnamon',
-		'xfce4',
-		'lxqt',
-		'i3',
-		'budgie',
-		'mate',
-		'deepin',
-		'enlightenment',
-	]
-
-	desktop = archinstall.generic_select(supported_desktops, 'Select your desired desktop environment: ', allow_empty_input=False, sort=True)
+	desktop = archinstall.Menu('Select your desired desktop environment', __supported__, skip=False).run()
 
 	# Temporarily store the selected desktop profile
 	# in a session-safe location, since this module will get reloaded
 	# the next time it gets executed.
-	if '_desktop_profile' not in archinstall.storage.keys():
+	if not archinstall.storage.get('_desktop_profile', None):
 		archinstall.storage['_desktop_profile'] = desktop
-
+	if not archinstall.arguments.get('desktop-environment', None):
+		archinstall.arguments['desktop-environment'] = desktop
 	profile = archinstall.Profile(None, desktop)
 	# Loading the instructions with a custom namespace, ensures that a __name__ comparison is never triggered.
 	with profile.load_instructions(namespace=f"{desktop}.py") as imported:
