@@ -26,7 +26,8 @@ def _prep_function(*args, **kwargs):
 	"""
 
 	supported_configurations = ['i3-wm', 'i3-gaps']
-	desktop = archinstall.generic_select(supported_configurations, 'Select your desired configuration: ', allow_empty_input=False, sort=True)
+
+	desktop = archinstall.Menu('Select your desired configuration', supported_configurations, skip=False).run()
 
 	# Temporarily store the selected desktop profile
 	# in a session-safe location, since this module will get reloaded
